package abc.meta;

import java.util.Collection;

import abc.aspectj.parse.AbcLexer;
import abc.aspectj.parse.LexerAction_c;
import abc.weaving.aspectinfo.GlobalAspectInfo;

/**
 * This is the abc extension for meta levels in AspectJ.
 * 
 * @author Eric Bodden
 */
public class AbcExtension extends abc.main.AbcExtension
{
    /** A customized global aspect info. */
    protected GlobalAspectInfo globalAspectInfo;

    /**
     * {@inheritDoc}
     */
    public void initLexerKeywords(AbcLexer lexer) {
        super.initLexerKeywords(lexer);
        // keyword for "meta" extension
        lexer.addGlobalKeyword("meta", new LexerAction_c(new Integer(abc.meta.parse.sym.META)));
    }

    /**
     * {@inheritDoc}
     */
    protected void collectVersions(StringBuffer versions)
    {
        super.collectVersions(versions);
        versions.append(" with Meta-Aspects " +
                        new Version().toString() +
                        "\n");
    }

    /**
     * {@inheritDoc}
     */
    public abc.aspectj.ExtensionInfo
            makeExtensionInfo(Collection jar_classes,
                              Collection aspect_sources)
    {
    	return new ExtensionInfo(jar_classes, aspect_sources);
    }
    
    /**
     * Create a customized global aspect info.
     */
    public GlobalAspectInfo getGlobalAspectInfo()
    {
        if (globalAspectInfo == null)
            globalAspectInfo = new abc.meta.weaving.aspectinfo.GlobalAspectInfo();

        return globalAspectInfo;
    }    
    
}
