package abc.meta;

import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import polyglot.ast.NodeFactory;
import polyglot.frontend.CupParser;
import polyglot.frontend.FileSource;
import polyglot.frontend.Job;
import polyglot.frontend.Parser;
import polyglot.frontend.Pass;
import polyglot.frontend.VisitorPass;
import polyglot.frontend.Pass.ID;
import polyglot.lex.Lexer;
import polyglot.types.TypeSystem;
import polyglot.util.ErrorQueue;
import abc.meta.ast.MetaNodeFactory;
import abc.meta.ast.MetaNodeFactory_c;
import abc.meta.parse.Grm;
import abc.meta.parse.Lexer_c;
import abc.meta.types.MetaTypeSystem_c;
import abc.meta.visit.DefaultMetaLevelAssigner;
import abc.meta.visit.MetaTypeBuilder;

/**
 * Extension info for the meta level extension to AspectJ.
 * 
 * @author Eric Bodden
 */
public class ExtensionInfo extends abc.aspectj.ExtensionInfo
{
    /** Pass ID for the pass assigning the default meta levels. */
    protected static final ID ASSIGN_DEFAULT_META_LEVELS = new ID("assign-default-meta-levels");
    
    static {
        // force Topics to load
        new Topics();
    }

    /**
     * @see abc.aspectj.ExtensionInfo#ExtensionInfo(Collection, Collection)
     */
    public ExtensionInfo(Collection jar_classes, Collection source_files)
    {
        super(jar_classes, source_files);
    }

    /**
     * {@inheritDoc}
     */
    public String compilerName() {
        return "meta-aspects";
    }

    /**
     * {@inheritDoc}
     */
    public Parser parser(Reader reader, FileSource source, ErrorQueue eq) {
        Lexer lexer = new Lexer_c(reader, source.name(), eq);
        Grm grm = new Grm(lexer, ts, nf, eq);
        return new CupParser(grm, source, eq);
    }

    /**
     * {@inheritDoc}
     */
    protected NodeFactory createNodeFactory() {
        return new MetaNodeFactory_c();
    }

    /**
     * {@inheritDoc}
     */
    protected TypeSystem createTypeSystem() {
        return new MetaTypeSystem_c();
    }
    
    /**
     * Add our custom passes.
     */
    public List passes(Job job) {
        List passes = super.passes(job);
        
        //replace the type builde by our own one
        List newPasses = new ArrayList();
        newPasses.add(new VisitorPass(Pass.BUILD_TYPES, job, new MetaTypeBuilder(job, ts, nf)));        
        replacePass(passes, Pass.BUILD_TYPES, newPasses);
        
        //insert the default meta level assigner
        newPasses = new ArrayList();
        newPasses.add(new VisitorPass(ASSIGN_DEFAULT_META_LEVELS, job, new DefaultMetaLevelAssigner((MetaNodeFactory) nf)));
        afterPass(passes, Pass.PARSE, newPasses);
        
        return passes;
    }
}
