package abc.meta;

import polyglot.util.ErrorInfo;
import polyglot.util.Position;
import abc.meta.weaving.aspectinfo.GlobalAspectInfo;

/**
 * This exception occurs when a too high metalevel is used in an aspect.
 * It is not to be thrown but rather only to be constructed.
 * The extception then emits its own message to the error queue.
 * 
 * @author Eric Bodden
 */
public class MetaLevelException extends RuntimeException {

    protected Position position;

    public MetaLevelException() {
        super();
        trace();
    }

    public MetaLevelException(Throwable cause) {
        super(cause);
        trace();
    }

    public MetaLevelException(Position position) {
        super();
        this.position = position;
        trace();
    }

    public MetaLevelException(String m) {
        super(m);
        trace();
    }

    public MetaLevelException(String m, Throwable cause) {
        super(m, cause);
        trace();
    }

    public MetaLevelException(String m, Position position) {
        super(m);
        this.position = position;
        trace();
    }

    private void trace() {
        //add the message to the trace
        abc.main.Main.v().error_queue.enqueue(ErrorInfo.SEMANTIC_ERROR,
                getMessage(), position);
        //notify the global aspect info that such an exception occured
        GlobalAspectInfo globalAspectInfo = (GlobalAspectInfo) abc.main.Main
                .v().getAbcExtension().getGlobalAspectInfo();
        globalAspectInfo.notifyMetaLevelError();
    }

    public Position position() {
        return position;
    }

}
