package abc.meta.ast;

import polyglot.ast.JL;
import polyglot.ext.jl.ast.AbstractDelFactory_c;
import abc.aspectj.ast.AJAbstractDelFactory_c;

/**
 * The delegate factory for the meta level extension.
 * This is not actually used internally but provided for the convenience of extenders.
 * @author Eric Bodden
 */
public class MetaDelFactory_c extends AJAbstractDelFactory_c implements
        MetaDelFactory {

    /** The next extension factory. */
    private final MetaDelFactory_c nextDelFactory;

    /**
     * Creates an extension factory with no next delegate factory.
     */
    public MetaDelFactory_c() {
        this(null);
    }
    
    /**
     * Creates an extension factory with a next delegate factory.
     * @param nextDelFactory the next delegate factory
     */
    public MetaDelFactory_c(MetaDelFactory_c nextDelFactory) {
        super(nextDelFactory);
        this.nextDelFactory = nextDelFactory;
    }
    
    /**
     * {@inheritDoc}
     */
    public JL delPCMetaTag() {
        JL d = delPCMetaTagImpl();

        if (nextDelFactory != null) {
            JL d2 = nextDelFactory.delPCMetaTag();
            d = composeDels(d, d2);
        }
        return postDelPCMetaTag(d);
    }
    
    /**
     * {@inheritDoc}
     */
    public JL delMetaTag() {
        JL d = delMetaTagImpl();

        if (nextDelFactory != null) {
            JL d2 = nextDelFactory.delMetaTag();
            d = composeDels(d, d2);
        }
        return postDelMetaTag(d);
    }
    
    /**
     * @return the delegate for meta tag pointcut nodes
     */
    protected JL delPCMetaTagImpl() {
        return delNodeImpl();
    }

    /**
     * @see AbstractDelFactory_c#postDelNode(del)
     */
    protected JL postDelPCMetaTag(JL del) {
        return postDelNode(del);
    }

    /**
     * @return the delegate for meta tag nodes
     */
    protected JL delMetaTagImpl() {
        return delNodeImpl();
    }

    /**
     * @see AbstractDelFactory_c#postDelNode(del)
     */
    protected JL postDelMetaTag(JL del) {
        return postDelNode(del);
    }
}
