package abc.meta.ast;

import polyglot.ast.Ext;
import polyglot.ext.jl.ast.AbstractExtFactory_c;
import abc.aspectj.ast.AJAbstractExtFactory_c;
import abc.meta.extension.ExtDefaultMetaLevelAssign_c;
import abc.meta.visit.DefaultMetaLevelAssigner;

/**
 * Extension factory for the meta level extension.
 * Each pointcut node is attached an extension which may assign a default
 * metalevel to that pointcut by surrounding it with a {@link PCMetaTag} pointcut
 * when visited by the {@link DefaultMetaLevelAssigner}.
 *
 * @author Eric Bodden
 */
public class MetaExtFactory_c extends AJAbstractExtFactory_c implements MetaExtFactory {
    
    /** The next extension factory. */
    private final MetaExtFactory_c nextExtFactory;

    /**
     * Creates an extension factory with no next extension factory.
     */
    public MetaExtFactory_c() {
        this(null);
    }
    
    /**
     * Creates an extension factory with a next extension factory.
     * @param nextExtFactory the next extension factory
     */
    public MetaExtFactory_c(MetaExtFactory_c nextExtFactory) {
        super(nextExtFactory);
        this.nextExtFactory = nextExtFactory;
    }
    
    //generally, we want to assign the default level to pointcuts
    
    /**
     * {@inheritDoc}
     */
    protected Ext extPointcutImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_POINTCUT_LEVEL);
    }

    //but only to basic pointcuts (i.e. pointcuts which hold no other child pointcuts)    

    /**
     * {@inheritDoc}
     */
    protected Ext extPCCflowImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }

    /**
     * {@inheritDoc}
     */
    protected Ext extPCCflowBelowImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }
    
    /**
     * {@inheritDoc}
     */
    protected Ext extPCBinaryImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }
    
    /**
     * {@inheritDoc}
     */
    protected Ext extPCNotImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }

    /**
     * {@inheritDoc}
     */
    protected Ext extPCNameImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }

    /**
     * @return the extension for meta tag nodes
     */
    protected Ext extPCMetaTagImpl() {
        return new ExtDefaultMetaLevelAssign_c(ExtDefaultMetaLevelAssign_c.ASSIGN_NO_LEVEL);
    }

    //this is just provided for the convenience of extenders
    
    /**
     * @return the extension for meta tag nodes
     */
    protected Ext extMetaTagImpl() {
        return extNodeImpl();
    }

    //boilerplate code follows
    
    /**
     * {@inheritDoc}
     */
    public Ext extPCMetaTag() {
        Ext e = extPCMetaTagImpl();

        if (nextExtFactory != null) {
            Ext e2 = nextExtFactory.extPCMetaTag();
            e = composeExts(e, e2);
        }
        return postExtPCMetaTag(e);
    }
    
    /**
     * {@inheritDoc}
     */
    public Ext extMetaTag() {
        Ext e = extMetaTagImpl();

        if (nextExtFactory != null) {
            Ext e2 = nextExtFactory.extMetaTag();
            e = composeExts(e, e2);
        }
        return postExtMetaTag(e);
    }
    
    /**
     * @see AbstractExtFactory_c#postExtNode(ext)
     */
    protected Ext postExtPCMetaTag(Ext ext) {
        return postExtNode(ext);
    }

    /**
     * @see AbstractExtFactory_c#postExtNode(ext)
     */
    protected Ext postExtMetaTag(Ext ext) {
        return postExtNode(ext);
    }
}
