package abc.meta.ast;

import java.util.List;

import polyglot.ast.TypeNode;
import polyglot.types.Flags;
import polyglot.util.Position;
import abc.aspectj.ast.AJNodeFactory;
import abc.aspectj.ast.AspectBody;
import abc.aspectj.ast.AspectDecl;
import abc.aspectj.ast.PerClause;
import abc.aspectj.ast.Pointcut_c;

/**
 * @author Eric Bodden
 */
public interface MetaNodeFactory extends AJNodeFactory {
    
    /** This meta level is assigned to aspects which have no meta tag annotation. */
    public static final int DEFAULT_ASPECT_LEVEL = 0;

	/**
     * Creates a meta tag pointcut, which is a decorator to a 
     * basic pointcut.
	 * @param delegate the basic pointcut which is decorated 
	 * @param level the meta level of the decorated pointcut
	 * @param pos the source code position
	 * @return the decorated pointcut
	 */
	public PCMetaTag PCMetaTag(Pointcut_c delegate, int level, Position pos);
    
    /**
     * Creates a meta tag for an aspect.
     * @param metaLevel the meta level of the decorated pointcut
     * @param pos the source code position
     * @return the decorated pointcut
     */
    public MetaTag MetaTag(int metaLevel, Position pos);

    /**
     * Same as {@link #AspectDecl(Position, boolean, Flags, String, TypeNode, List, PerClause, AspectBody, int)}
     * but a default level is provided (for non-meta aspects).
     */
    public AspectDecl AspectDecl(Position pos, boolean is_privileged, Flags flags, String name, TypeNode superClass, List interfaces, PerClause per, AspectBody body);  

    /**
     * Same as {@link AJNodeFactory#AspectDecl(Position, boolean, Flags, String, TypeNode, List, PerClause, AspectBody)}
     * but with an additional meta level for the aspect.
     */
    public AspectDecl AspectDecl(Position pos, boolean is_privileged, Flags flags, String name, TypeNode superClass, List interfaces, PerClause per, AspectBody body, MetaTag metaLevelTag);
	
}
