package abc.meta.ast;

import java.util.List;

import polyglot.ast.Receiver;
import polyglot.ast.TypeNode;
import polyglot.types.Flags;
import polyglot.util.Position;
import abc.aspectj.ast.AJDelFactory;
import abc.aspectj.ast.AJExtFactory;
import abc.aspectj.ast.AJNodeFactory_c;
import abc.aspectj.ast.AspectBody;
import abc.aspectj.ast.AspectDecl;
import abc.aspectj.ast.PCName;
import abc.aspectj.ast.PerClause;
import abc.aspectj.ast.Pointcut;
import abc.aspectj.ast.Pointcut_c;

/**
 * Node factory for the meta level extension.
 * 
 * @author Eric Bodden
 */
public class MetaNodeFactory_c extends AJNodeFactory_c implements MetaNodeFactory {

    /**
     * Creates a new node factory with the default extension factory for the meta level extension
     * and the default delegation factory for plain AspectJ.
     */
    public MetaNodeFactory_c() {
        super(new MetaExtFactory_c(), new MetaDelFactory_c());
    }
    
    /**
     * Creates a new node factory with a custom extension factory but the default
     * delegate factory for plain AspectJ.
     * This is not used internally and only exists for the convenience of extenders.
     * @param extFactory a custom extension factory
     */
    public MetaNodeFactory_c(MetaExtFactory extFactory) {
        super(extFactory);
    }
    
    /**
     * Creates a new node factory with a custom extension and delegate factory.
     * This is not used internally and only exists for the convenience of extenders.
     * @param extFactory a custom extension factory
     * @param delFactory a custom delegate factory
     */
    public MetaNodeFactory_c(MetaExtFactory extFactory, MetaDelFactory delFactory) {
        super(extFactory, delFactory);
    }


    /**
     * This pointcut is added as a decorator to all basic pointcuts.
     * It holds the meta level.
     * <br>
     * We cannot simply use an extension object, because we need to change
     * {@link Pointcut#makeAIPointcut()}, which is often directly accessed.
     */
    public PCMetaTag PCMetaTag(Pointcut_c delegate, int level, Position pos) {
        PCMetaTag n = new PCMetaTag_c(delegate, level, pos);
        n = (PCMetaTag)n.ext(((MetaExtFactory) extFactory()).extPCMetaTag());
        n = (PCMetaTag)n.del(((MetaDelFactory) delFactory()).delPCMetaTag());
        return n;
    }
    
    public MetaTag MetaTag(int metaLevel, Position pos) {
        MetaTag n = new MetaTag_c(metaLevel, pos);
        n = (MetaTag)n.ext(((MetaExtFactory) extFactory()).extMetaTag());
        n = (MetaTag)n.del(((MetaDelFactory) delFactory()).delMetaTag());
        return n;
    }
    
    /**
     * Need special aspect declarations which can hold a meta level.
     * Those ones here have default level.
     */
    public AspectDecl AspectDecl(Position pos, boolean is_privileged, Flags flags, String name, TypeNode superClass, List interfaces, PerClause per, AspectBody body) {
        return AspectDecl(pos, is_privileged, flags, name, superClass, interfaces, per, body, MetaTag(DEFAULT_ASPECT_LEVEL,pos));
    }

    /**
     * Need special aspect declarations which can hold a meta level.
     */
    public AspectDecl AspectDecl(Position pos, boolean is_privileged, Flags flags, String name, TypeNode superClass, List interfaces, PerClause per, AspectBody body, MetaTag metaTag) {
        AspectDecl n = new AspectDecl_c(pos, is_privileged, flags, name, superClass, interfaces, per, body, metaTag);
        n = (AspectDecl)n.ext(((AJExtFactory) extFactory()).extAspectDecl());
        n = (AspectDecl)n.del(((AJDelFactory) delFactory()).delAspectDecl());
        return n;
    }
    
    /**
     * Need a stpecial treatment for named pointcuts in order to present 
     * error messages at the right position.
     */
    public PCName PCName(Position pos, Receiver target, String name, List args) {
        PCName n = new PCName_c(pos, target, name, args);
        n = (PCName)n.ext(((AJExtFactory) extFactory()).extPCName());
        n = (PCName)n.del(((AJDelFactory) delFactory()).delPCName());
        return n;
    }
	
}
