package abc.meta.extension;

import abc.aspectj.ast.Pointcut;
import abc.meta.ast.MetaNodeFactory;

/**
 * This polyglot node extension assigns default meta level nodes to pointcuts.
 * However, it only does so, when {@link #ASSIGN_POINTCUT_LEVEL} is chosen.
 *
 * @author Eric Bodden
 */
public interface ExtDefaultMetaLevelAssign {

    /** If this is given to the constructor, no level is assigned to the extended node. */
    public static final int ASSIGN_NO_LEVEL = 0;

    /** If this is given to the constructor, the default pointcut level is assigned to the extended node. */
    public static final int ASSIGN_POINTCUT_LEVEL = 1;
    
    /** The default meta level (for classes). */
    public static final int DEFAULT_LEVEL_STANDARD = 0;
    
    /**
     * Assigns the {@link #DEFAULT_LEVEL_STANDARD} to nodes for which {@link #ASSIGN_POINTCUT_LEVEL} was selected.
     * @param nf the node factory
     * @return the possibly augmented node
     */
    public Pointcut assignDefaultMetaLevel(MetaNodeFactory nf);
    
}