package abc.meta.extension;

import polyglot.ast.Ext;
import polyglot.ext.jl.ast.Ext_c;
import abc.aspectj.ast.Pointcut;
import abc.aspectj.ast.Pointcut_c;
import abc.meta.ast.MetaNodeFactory;

/**
 * This polyglot node extension assigns default meta level nodes to pointcuts.
 * However, it only does so, when {@link ExtDefaultMetaLevelAssign#ASSIGN_POINTCUT_LEVEL} is chosen.
 *
 * @author Eric Bodden
 */
public class ExtDefaultMetaLevelAssign_c extends Ext_c implements Ext, ExtDefaultMetaLevelAssign {
    
    /** One of {@link ExtDefaultMetaLevelAssign#ASSIGN_NO_LEVEL},
     * {@link ExtDefaultMetaLevelAssign#ASSIGN_POINTCUT_LEVEL} */
    protected final int type;
    
    /**
     * Creates a new node extension.
     * @param type one of {@link ExtDefaultMetaLevelAssign#ASSIGN_NO_LEVEL},
     * {@link ExtDefaultMetaLevelAssign#ASSIGN_POINTCUT_LEVEL}
     */
    public ExtDefaultMetaLevelAssign_c(int type) {
        this.type = type;
    }
    
    /**
     * {@inheritDoc}
     */
    public Pointcut assignDefaultMetaLevel(MetaNodeFactory nf) {
        Pointcut_c pc = (Pointcut_c) node;
        
        switch(type) {
        case ASSIGN_NO_LEVEL:
            return pc;
        case ASSIGN_POINTCUT_LEVEL:
            return nf.PCMetaTag(pc, DEFAULT_LEVEL_STANDARD, pc.position());
            default:
                throw new RuntimeException("Internal error - wrong type set");
        }
    }

}
