package abc.meta.visit;

import polyglot.ast.Node;
import polyglot.visit.NodeVisitor;
import abc.aspectj.ast.Pointcut;
import abc.meta.ast.MetaNodeFactory;
import abc.meta.ast.PCMetaTag;
import abc.meta.extension.ExtDefaultMetaLevelAssign;

/**
 * Visitor assigning default meta levels to poitncuts.
 * This is done by adding a decorator node of type {@link PCMetaTag} to each
 * (basic) pointcut which is not already decorated with such a node.
 *
 * @author Eric Bodden
 */
public class DefaultMetaLevelAssigner extends NodeVisitor {
    
    /** The node factory. */
    protected MetaNodeFactory nf;
    
    /**
     * Creates a new visitor.
     * @param nf the node factory
     */
    public DefaultMetaLevelAssigner(MetaNodeFactory nf) {
        super();
        this.nf = nf;
    }

    /**
     * Assigns the default meta level for all pointcuts (except meta pointcuts of course).
     */
    public Node leave(Node old, Node n, NodeVisitor v) {
        if((n instanceof Pointcut) &&!(n instanceof PCMetaTag)) {
            ExtDefaultMetaLevelAssign defaultLevelAssign =  (ExtDefaultMetaLevelAssign) n.ext();
            //the following has no effect for meta pointcuts and compound pointcuts (which
            //hold another pointcut as child, such as Boolean pointcuts or cflow);
            //this is ensured by attaching custom extension nodes in
            //MetaExtFactory_c
            n = defaultLevelAssign.assignDefaultMetaLevel(nf);
        }        
        return super.leave(old, n, v);
    }

}
