package abc.meta.visit;

import java.util.Stack;

import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.frontend.Job;
import polyglot.types.Flags;
import polyglot.types.ParsedClassType;
import polyglot.types.SemanticException;
import polyglot.types.TypeSystem;
import polyglot.util.Position;
import polyglot.visit.NodeVisitor;
import abc.aspectj.visit.AJTypeBuilder;
import abc.meta.ast.AspectDecl;
import abc.meta.types.AspectType;

/**
 * An extended version of the type builder, assigning the meta level to the type nodes
 * for aspect declarations after they have been created.
 * @author Eric Bodden
 */
public class MetaTypeBuilder extends AJTypeBuilder {

    /** 
     * Stack to hold the innermost aspect node. Unfortunately, the AJTypeBuilder is no
     * Contextvisitor, so we have to do this manually.
     */
    protected Stack currAspect;
    
	/**
	 * @see AJTypeBuilder#AJTypeBuilder(Job, TypeSystem, NodeFactory)
     */
	public MetaTypeBuilder(Job job, TypeSystem ts, NodeFactory nf) {
		super(job, ts, nf);
        currAspect = new Stack();
	}

    /**
     * {@inheritDoc}
     */
    public NodeVisitor enter(Node n) {
        //store surrounding aspect
        if(n instanceof AspectDecl) {
            currAspect.push(n);
        }
        return super.enter(n);
    }
    
    /**
     * {@inheritDoc}
     */
    public Node leave(Node old, Node n, NodeVisitor v) {
        //pop surrounding aspect
        if(n instanceof AspectDecl) {
            Node former = (Node) currAspect.pop();
            //assert that we pop the correct thing
            assert ((AspectDecl) former).name().equals(((AspectDecl) n).name());  
        }
        return super.leave(old, n, v);
    }
    
	/**
	 * {@inheritDoc}
	 */
	protected ParsedClassType newAspect(Position pos, Flags flags, String name, int perKind) 
	  throws SemanticException{
        //get the original type
		ParsedClassType res = super.newAspect(pos, flags, name, perKind);
		AspectType at = (AspectType) res;
        
		//set its meta level based on the AST node
        AspectDecl currAsp = (AspectDecl) currAspect.peek();
        at.setMetaLevel(currAsp.getMetaLevel());
		return res;		
	}

	
}
