package abc.meta.weaving.aspectinfo;

import polyglot.types.SemanticException;

/**
 * A custom version of the global aspect info, which gets notified if type checking errors
 * occur during weaving. If this happens, it throws an appropriate semantic exception.
 *
 * @author Eric Bodden
 */
public class GlobalAspectInfo extends abc.weaving.aspectinfo.GlobalAspectInfo {
    
    /** <code>true</code> if a meta level error occurred */
    protected boolean hasSemanticErrors = false; 
    
    /**
     * Signal that a metalevel error occurred.
     * Weaving will be aborted after having computed the advice lists.
     */
    public void notifyMetaLevelError() {
        this.hasSemanticErrors = true;
    }
    
    /**
     * Aborts compilation if meta level errors have occurred.
     */
    public void computeAdviceLists() throws SemanticException {
        super.computeAdviceLists();
        if(hasSemanticErrors) {
            throw new SemanticException("There were meta level errors. Compilation aborted.");
        }
    }

}
