package abc.meta.weaving.aspectinfo;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import abc.weaving.aspectinfo.Aspect;
import abc.weaving.aspectinfo.Pointcut;

import polyglot.util.Position;

/**
 * Modified version of the aspect info for named pointcuts.
 * When we do typechecking during inlining, we always want to know the currently outermost
 * surrounding named pointcut (if any) in order to give accurate position information.
 *
 * @author Eric Bodden
 */
public class PointcutRef extends abc.weaving.aspectinfo.PointcutRef {

    /** The currently outermost named pointcut being examined (if any). */
    private static PointcutRef currentOuterMostRef;
    
    /**
     * @see abc.weaving.aspectinfo.PointcutRef#PointcutRef(Object, Map, List, Position, boolean)
     */
    public PointcutRef(Object decl_key, Map decl_map, List args, Position pos, boolean hardref) {
        super(decl_key, decl_map, args, pos, hardref);
    }
    
    /**
     * Stores this object if it is the outermost one being inlined and removes it after inlining.
     */
    public Pointcut inline(Hashtable renameEnv, Hashtable typeEnv, Aspect context, int cflowdepth) {
        //store the object
        if(currentOuterMostRef == null) {
            currentOuterMostRef = this;
        }
        //inline everything in between
        Pointcut res = super.inline(renameEnv, typeEnv, context, cflowdepth);
        //remove us again
        if(currentOuterMostRef == this) {
            currentOuterMostRef = null;
        }
        return res;
    }

    /**
     * @return the current outermost named pointcut during inlining
     */
    public static PointcutRef getCurrentOuterMostRef() {
        return currentOuterMostRef;
    }
    

}
